/****************************************************************************
*      Project: Blue Box
*         File: dispfnct.c
*      Revison: 0.01 12/29/97    rjs   Created
*  Description: This file contains all functions used by the parameter 
*				display routines.
*****************************************************************************/



#pragma memory=constseg(B14_CONSTANTS)

/* these are note frequencies for octave "10" mult by 65536 */
UINT32 note_ref[12] =
{
	1097337132L,	/* C   */
	1162588193L,	/* C#  */
	1231719284L,	/* D   */
	1304961123L,	/* D#  */
	1382558149L,	/* E   */
	1464769334L,	/* F   */
	1551869051L,	/* F#  */
	1644147986L,	/* G   */
	1741914112L,	/* G#  */
	1845493715L,	/* A   */
	1955232481L,	/* A#  */
	2071496654L,	/* B   */
};


#pragma memory=default


#define TEST_THIS_OUT
#ifdef TEST_THIS_OUT
#define temp_buf_2 (&temp_buf[37])
#endif



///////////////////////////////////
// Lexichip Display Functions

void LEXDisplaySPFreq(EDIT_OBJECT *GroupPtr, UINT8	Mode);
void LEXDisplayDPFreq(EDIT_OBJECT *GroupPtr, UINT16 Value, UINT8	Mode);

//
//	LEXDisplayScale	-	Display Room Size in Meters
//
//**************************************************************************/

void LEXDisplayScale(EDIT_OBJECT	*GroupPtr,
		     UINT8		Mode)
{
    UINT16	Value;
    UINT16	Meters;
	UINT8 	buf[7];
	UINT8	position;

    
    // If it is less than 1khz, we'll output the hertz value //
    ///////////////////////////////////////////////////////////
	if (Mode == LONG_MODE)	
		position = GroupPtr->position-7;
 	else
		position = GroupPtr->position;

    
 	set_position(position, Mode);


    Value = *GroupPtr->dsp_param_ptr;

    Meters = (Value >> 1) + 4;
    create_decimal_number(Meters, buf);
    if(Value & 1)
		string_cat(buf, ".5");
    else
		string_cat(buf, ".0");


	if (Mode == LONG_MODE)	
	{
		write_string_rj(buf, Mode);
	 	set_position(GroupPtr->position, SHORT_MODE);
		write_string_rj( " Meters", SHORT_MODE);
	}
	else
	{
		string_cat(buf, "M");
		write_string_rj(buf, Mode);
	}
/*    display_decimal_number_in_mode(Meters, Mode);
    if(Value & 1)
	write_string(".5M", Mode);
    else
	write_string(".0M", Mode);
*/
}



//
//	LEXDisplayTrebDecay	-	Display the TrebDecay Xover frequency
//
//	We just add in the bias and call the standard DisplayXover function
//
//**************************************************************************/

void LEXDisplayTrebDecay(EDIT_OBJECT	*GroupPtr,
			 UINT8		Mode)
{
    UINT16	Value;

    Value = *GroupPtr->dsp_param_ptr + LEX_TREB_BIAS;

    if(0xffff == xover_table[Value].freq)
	{
	write_string_rj("Flat", Mode);
	}
    else
	{
	LEXDisplayDPFreq(GroupPtr, Value, Mode);
	}
//    LEXDisplayDPFreq(GroupPtr, Value, Mode);
}

//
//	GENDisplayXover	-	Display the Xover frequency
//
//	Output, not the current DisplayValue, but the current Xover
//	frequency from the xover_freq table.
//
//**************************************************************************/

void LEXDisplayXover(EDIT_OBJECT	*GroupPtr,
		     UINT8		Mode)
{
    UINT16	Value;

    Value = *GroupPtr->dsp_param_ptr;

    if(0xffff == xover_table[Value].freq)
	{
	write_string_rj("Full", Mode);
	//write_string_rj("Off", Mode);
	}
    else
	{
	LEXDisplayDPFreq(GroupPtr, Value, Mode);
	}
}


//
//	LEXDisplayFrequency	-	Display a formatted frequency value
//
//**************************************************************************/

void LEXDisplayFrequency(EDIT_OBJECT	*GroupPtr,
			 FREQUENCY	*FPtr,
			 UINT16		freq,
			 UINT8		Mode)
{
    int		Value;
    char	buffer[8];
    char	*dest = buffer;
    char	position;

    
    // If it is less than 1khz, we'll output the hertz value //
    ///////////////////////////////////////////////////////////

    if (Mode == LONG_MODE)	
	position = GroupPtr->position-6;
    else
	position = GroupPtr->position;
    set_position(position, Mode);

    if(1000 > freq)
	{
	create_decimal_number(freq, buffer);
	write_string_rj(buffer, Mode);
/*	display_decimal_number_in_mode(freq, Mode);
	write_string( "hz", Mode);
*/
	}


    else
	{
	Value = FPtr->khertz;
	if (Value >= 10)
	    {
	    if(Value >= 20)
		{
		Value -= 20;
		*dest++ = '2';
		}
	    else
		{
		Value -= 10;
		*dest++ = '1';
		}
	    }
    
	*dest++ = '0' + Value;
	*dest++ = '.';
	*dest++ = FPtr->hertz;
	*dest++ = 'K';
	*dest = 0;
	write_string_rj(buffer, Mode);
	}

	if (Mode == LONG_MODE)	
	{
	 	set_position(GroupPtr->position, SHORT_MODE);
		write_string_rj( " Hertz", SHORT_MODE);
	}

}

//
//	LEXDisplaySPFreq	-	Display a double precision filter
//
//**************************************************************************/

void LEXDisplaySPFreq(EDIT_OBJECT	*GroupPtr,
		      UINT8		Mode)
{

    // Invoke the common formatting code //
    ///////////////////////////////////////
    
    LEXDisplayFrequency(GroupPtr,
			&SPFrequencies[*GroupPtr->dsp_param_ptr + 1],
			1001,
			Mode);
}


//
//	LEXDisplayDPFreq	-	Display a double precision filter
//
//**************************************************************************/

void LEXDisplayDPFreq(EDIT_OBJECT	*GroupPtr,
		      UINT16	Value,
		      UINT8	Mode)
{
    
    // Invoke the common formatting code //
    ///////////////////////////////////////
    
    LEXDisplayFrequency(GroupPtr,
			&DPFrequencies[Value],
			xover_table[Value].freq,
			Mode);
}


//
//	LEXDisplayBassRT	-	Display the Bass Boost factor
//
//	Output, not the current DisplayValue, but the current BassFactor
//	string from the bassmult table.
//
//**************************************************************************/

void LEXDisplayBassRT(EDIT_OBJECT	*GroupPtr,
		      BYTE		Mode)
{
	UINT8 	buf[7];

    copy_string(BassRtTable[*GroupPtr->dsp_param_ptr].String, buf);
    string_cat(buf, "X");
	write_string_rj(buf, Mode);

/*   write_string(BassRtTable[*GroupPtr->dsp_param_ptr].String, Mode);
    write_string("X", Mode);
*/
}



//
//	LEXDisplayMidRT	-	Display the MidRange ReverbTime
//
//
//**************************************************************************/

void LEXDisplayMidRT(EDIT_OBJECT	*GroupPtr,
		     BYTE		Mode)
{
    UINT8		Index;
    MSEC		value;
    UINT8		Limit;
    char		ch;
    MINT		FracCount;
    char		buffer[8];
    char		*dest = buffer;
    

    value = MidRtTimes[*GroupPtr->dsp_param_ptr];
#ifdef EMBEDDED_Z80_CODE
    if(ReverbData.RVB_Link)
	value = ((long)value * (long)ReverbData.RVB_SizeRatio) /
		MAX_SIZE_RATIO;
#endif

    value += (MSEC_PER_SECOND/20);
    

    // Ok, generate the character for 10s of seconds. If there isn't one, //
    // we can have an extra digit to the right of the decimal point.      //
    ////////////////////////////////////////////////////////////////////////
    
    if(value >= 10000)
	{
	FracCount = 1;
	ch = '0';
	while(value >= 10000)
	    {
	    ch++;
	    value -= 10000;
	    }
	*dest++ = ch;
	}
    else
	{
	FracCount = 2;
	}
    

    // Generate seconds //
    //////////////////////
    
    ch = '0';
    while(value >= 1000)
	{
	value -= 1000;
	ch++;
	}
    *dest++ = ch;
    *dest++ = '.';
    
    
    // Now, output the milliseconds. We'll have either two or three digits //
    /////////////////////////////////////////////////////////////////////////
    

    for(Index = 0; Index < FracCount; Index++)
	{
	Limit = Limits[Index];
	ch = '0';
	while(value > Limit)
	    {
	    value -= Limit;
	    ch++;
	    }
	*dest++ = ch;
	}
    *dest++ = 's';
    *dest++ = 0;
    write_string_rj(buffer, Mode);
}



//
//	LEXDisplayPercent50	-	Display percentages (50 ticks)
//
//
//**************************************************************************/

void LEXDisplayPercent50(EDIT_OBJECT	*GroupPtr,
			 UINT8		Mode)
{
    UINT16		Data;
	UINT8 	buf[7];
    
    Data = *GroupPtr->dsp_param_ptr << 1;
    create_decimal_number(Data, buf);
	string_cat(buf, "%");
	write_string_rj(buf, Mode);

/*    display_decimal_number_in_mode(Data, Mode);
    write_string( "% ", Mode);
*/
}


//
//      LEXDisplaySpread	-	Display the current Spread setting
//
//	The spread display is effected by Link and Size. If Link is on (TRUE),
//	then the display is multiplied by (currsize/maxsize).
//
//**************************************************************************/

void LEXDisplaySpread(EDIT_OBJECT	*GroupPtr,
		      UINT8		Mode)
{
    UINT16	MaxFactor;
    UINT16	Value;
	UINT8 	buf[7];

    Value = *GroupPtr->dsp_param_ptr;
#ifdef EMBEDDED_Z80_CODE
	if(ReverbData.RVB_Link)
	{
		MaxFactor = MAX_SIZE_RATIO << SCALE_SHIFT;
		Value = (UINT32)((UINT32)Value * ReverbData.RVB_SizeFactor) / MaxFactor;
	}
#endif
	create_decimal_number(Value, buf);
	write_string_rj(buf, Mode);
/*    display_decimal_number_in_mode(Value, Mode);*/
}


//
//	LEXDisplaySlope	-	Display a slope value
//
//	Slope displays are very simple. Bias the value so that the midpoint
//	of the range becomes zero and then multiply by four. Thus a Param
//	with a range of 0..32 becomes -16..16 and displays as -64%..64%.
//
//**************************************************************************/

void LEXDisplaySlope(EDIT_OBJECT	*GroupPtr,
		     UINT8		Mode)
{
/*    char	Sign;*/
    BYTE	Value;
	UINT8 	buf[7];
    
    Value = *GroupPtr->dsp_param_ptr;
    if(Value < 16)
	{
		buf[0] = '-';
		Value = 16 - Value;
	}
    else
	{
		buf[0] = '+';
		Value -= 16;
	}
    create_decimal_number(Value, &buf[1]); /* keep the sign character */
	write_string_rj(buf, Mode);

/*    write_char(Sign, Mode);
    display_decimal_number_in_mode(Value, Mode);
*/
}


//
//      GENDisplayDuration	-	Show duration of Inverse
//
//	The basic delay through the Inverse algorithm is 140ms. Each tick
//	of the knob adds 5 ms.
//
//**************************************************************************/

void LEXDisplayDuration(EDIT_OBJECT	*GroupPtr,
			UINT8		Mode)
{
    UINT16	Value;
	UINT8 	buf[7];

    Value = *GroupPtr->dsp_param_ptr;
    create_decimal_number(140 + (Value * 5), buf);
	string_cat(buf, "ms");

	write_string_rj(buf, Mode);

/*
    display_decimal_number_in_mode(140 + (Value * 5), Mode);
    write_string("ms", Mode);
*/
}


//
//	GENDisplayDPLevel	-	Display the Reverb Level in DB
//
//	Lookup in one of two table depending on function type to 
//	convert from value to a dB number. One table has a range 
//	of 16 the other 26. All source data is passed to 
//	param_decimal1_disp() which does all of the work. Units 
//	of table are in k, i.e. entry of 1000 maps to a display 
//	of -1.0 dB. The course control, levelog_disp() has a range 
//	of 0 dB to -48 dB and the fine control, splevelog_disp()
//	has a range of 0 to -24 dB.
//
//***************************************************************************/

void LEXDisplayDPLevel(EDIT_OBJECT	*GroupPtr,
		       BYTE		Mode)
{
    UINT8	Value;
    char	buffer[8];
    char	*dest = buffer;
    
    
    Value = *GroupPtr->dsp_param_ptr;

    if(0 == Value)
	{
	write_string_rj("Off ", Mode);
	}
    else if(Value == GroupPtr->param_units.max_value)
	{
	write_string_rj("Full", Mode);
	}
    else
	{
	Value <<= 1;
	*dest++ = '-';
	*dest++ = levelog_db[Value++];
	*dest++ = levelog_db[Value++];
	*dest   = 0;
	string_cat(buffer, "dB");
	write_string_rj(buffer, Mode);
	}

}


/////////////////////////////////
/////////////////////////////////

//void display_bipolar_byte(EDIT_OBJECT *group_ptr, char *buf_ptr)
void display_bipolar_byte(BYTE data, char *buf_ptr)
{
	//BYTE data = *group_ptr->dsp_param_ptr;

	if (data == 0)
	{
		buf_ptr[0] = '0';
		buf_ptr[1] = 0;
		return;
	}			

	if (data & 0x80)
	{
		buf_ptr[0] = '-';
		data = (0xff - data) + 1;
	}
	else
		buf_ptr[0] = '+';

	buf_ptr[1] = 0; /* null terminate */

	create_decimal_number((UINT16) data, &buf_ptr[1] ); /* add number after sign */
}

void display_bipolar_word(EDIT_OBJECT *group_ptr, BYTE mode)
{
	UINT16 data;
	UINT16 *word_ptr;
	
	word_ptr = (UINT16 *) group_ptr->dsp_param_ptr;
	data  = *word_ptr;

	if (data & 0x8000)
	{
		temp_buf[0] = '-';
		data = (0xffff - data) + 1;
	}
	else
		temp_buf[0] = '+';

	temp_buf[1] = 0; /* null terminate */

	create_decimal_number( data, &temp_buf[1] ); /* add number after sign */
	write_string_rj( temp_buf, mode );
}

void display_in_options_no_units( EDIT_OBJECT *group_ptr, BYTE mode)
{
	display_decimal_number_in_mode((UINT16) *group_ptr->dsp_option_ptr, mode);
}

void display_in_bipolar_no_units( EDIT_OBJECT *group_ptr, BYTE mode)
{
	display_bipolar_byte(*group_ptr->dsp_param_ptr, temp_buf);
	write_string_rj( temp_buf, mode );
}

void display_in_signed_bargraph_units( EDIT_OBJECT *group_ptr, BYTE mode)
{
	//BYTE *ptr;

#ifdef EMBEDDED_Z80_CODE	
	// This is somewhat of a kluge. This keeps getting set every time the 
	// the display gets redrawn. Note that a function is placed in the aux_function_ptr
	// by the set up below. Any changes to the "pointed to" variable update
	// the bargraph display. THe function gets cleared when a new parameter is 
	// selected or the effect_edit_menu is exited.
//	if (group_ptr->effect_desc.type == FX_1_EFFECT)
//		ptr = &dsp_params.data[FX_1_EFFECT][20];
//	else
//		ptr = &dsp_params.data[FX_2_EFFECT][20];
//	set_up_vertical_bargraph(ptr);
//
//	write_char_to_buf_position(4, 19);
#endif

	display_bipolar_byte(*group_ptr->dsp_param_ptr, temp_buf);
	write_string_rj( temp_buf, mode );
}

void display_note(UINT8 note, UINT8 position)
{
	set_position(position, SHORT_MODE);

	if (note < 12)
		write_string( (char *) cromatic_note_strings[note], SHORT_MODE );
	else
		write_string( "  ", SHORT_MODE );
}


/* this unit is only used for TestTone */
/* use 	flexy_byte_z80 to toggle optional frequency display */
void display_cromatic_notes(EDIT_OBJECT *group_ptr, BYTE mode)
{
	BYTE note = *group_ptr->dsp_param_ptr;
	BYTE octave = 0;

	while(note > 11)
	{
		++octave;
		note -= 12;
	}		

	if (flexy_byte_z80 & FLEXY_FREQ)
		create_decimal_number( (note_ref[note] >> (16 + (10-octave)))+1, temp_buf);
	else
	{
		BYTE *buf_ptr = copy_string( cromatic_note_strings[note], temp_buf );
		create_decimal_number( octave, buf_ptr);
	}

	write_string_rj( temp_buf, mode );
}


void get_last_char_offset(char *string_ptr, BYTE *offset_ptr)
{
	*offset_ptr = 0;
	while ( *string_ptr )
	{
		++*offset_ptr;
		++string_ptr;
	}		
}

void display_string_units(EDIT_OBJECT *group_ptr, BYTE mode, char **strings_ptr)
{
	char *string_ptr;

	string_ptr = strings_ptr[*group_ptr->dsp_param_ptr];

	write_string_rj( string_ptr, mode );
}


void display_wah_type_units(EDIT_OBJECT *group_ptr, BYTE mode)
{
	write_string_rj( (char *) wah_type_strings[*group_ptr->dsp_option_ptr], mode );
}

void display_diatonic_input_units(EDIT_OBJECT *group_ptr, BYTE mode)
{
	write_string_rj( (char *) noise_gate_enable_strings[*group_ptr->dsp_option_ptr], mode );
}


void display_in_feedback_insert_units(EDIT_OBJECT *group_ptr, BYTE mode)
{
	write_string_rj( (char *) fb_insert_point_strings[*group_ptr->dsp_option_ptr], mode );
}

void display_in_time_unit_units(EDIT_OBJECT *group_ptr, BYTE mode)
{
	write_string_rj( (char *) time_unit_strings[*group_ptr->dsp_option_ptr], mode );
}

void display_envelope_source_units(EDIT_OBJECT *group_ptr, BYTE mode)
{
	write_string_rj( (char *) envelope_source_strings[*group_ptr->param_ptr], mode );
}

void display_in_phase_units(EDIT_OBJECT *group_ptr, BYTE mode)
{
	write_string_rj( (char *) phase_strings[*group_ptr->dsp_param_ptr], mode );
}

void display_in_mix_or_dry_units( EDIT_OBJECT *group_ptr, BYTE mode)
{
	/* here we have the problem of determining from two alternative settings
	 * before, the dsp_option_ptr points to the actual mix value
	 * after, it points to an intermediate setting; either 101 or 102
	 * from 0 to 100 it represents mix setting, 101 represents mix units choice,
	 * 102 represents dry level units choice, 165 to 255 represents dry level setting
	 */
	write_string_rj( (char *) mixdry_unit_strings[(*group_ptr->dsp_option_ptr >= 102)], mode );
}


#pragma warnings=off  /* don't complain about unused arguments */

void display_in_dump_units(EDIT_OBJECT *group_ptr, BYTE mode)
{
	write_string_rj( "Dump", mode );
}
	
void display_in_pan_units( EDIT_OBJECT *group_ptr, BYTE mode)
{
	BYTE data = *group_ptr->dsp_param_ptr;

	if (data == 0)
	{
		if (mode == LONG_MODE)	
			write_string_rj( "Center", mode);
		else
			write_string_rj( "C", mode);
		return;
	}

	if (data & 0x80) /* left? */
	{
		data = (0xff - data) + 1;
		create_decimal_number((UINT16) data, temp_buf);
		if (mode == LONG_MODE)	
	 		string_cat( temp_buf, " Left");
		else
	 		string_cat( temp_buf, "L");
		write_string_rj( temp_buf, mode);
	}
	else
	{
		create_decimal_number((UINT16) data, temp_buf);
		if (mode == LONG_MODE)	
	 		string_cat( temp_buf, " Right");
		else
	 		string_cat( temp_buf, "R");
		write_string_rj( temp_buf, mode);
	}
}

void display_in_bal_units( EDIT_OBJECT *group_ptr, BYTE mode)
{
	BYTE data = *group_ptr->dsp_param_ptr;

	if (data == 0)
	{
		write_string_rj( "0", mode);
		return;
	}

	display_in_bipolar_no_units(group_ptr, mode);
}


void display_in_no_units( EDIT_OBJECT *group_ptr, BYTE mode)
{
	BYTE *data_ptr;

#ifdef EMBEDDED_Z80_CODE	
	if (group_ptr->dsp_param_ptr) 
#endif
		data_ptr = group_ptr->dsp_param_ptr;
#ifdef EMBEDDED_Z80_CODE	
	else if (cur_choices.compare_mode == EDITED_PROG)
		data_ptr = group_ptr->param_ptr;
	else
		data_ptr = group_ptr->original_param_ptr;
#endif		
	display_decimal_number_in_mode( *data_ptr, mode ); 
}


static void display_in_x_units( EDIT_OBJECT *group_ptr, BYTE mode, BYTE type)
{
	BYTE position;

	position = group_ptr->position;
	group_ptr->position = position - 2;

	set_position(position, SHORT_MODE);
	write_string_rj( "ms",SHORT_MODE );

	set_position(group_ptr->position, mode);
	if (!type)
		display_in_no_units( group_ptr, mode);
	else
		display_in_options_no_units( group_ptr, mode);
}

void display_in_pdly_units( EDIT_OBJECT *group_ptr, BYTE mode)
{
	display_in_x_units( group_ptr, mode, 0);
}

void display_in_optimize_units( EDIT_OBJECT *group_ptr, BYTE mode)
{
	display_in_x_units( group_ptr, mode, 1);
}

void display_in_ff_off_no_units( EDIT_OBJECT *group_ptr, BYTE mode)
{
	if (*group_ptr->dsp_param_ptr == 0xff)
		write_string_rj( "None" /*"Off"*/, mode );
	else
	{
		copy_string( "CC", temp_buf );
		create_decimal_number((UINT16) *group_ptr->dsp_param_ptr, &temp_buf[2]);
		write_string_rj( temp_buf, mode);
	}
}


void display_in_no_units_word( EDIT_OBJECT *group_ptr, BYTE mode)
{
	UINT16 *word_ptr;

	word_ptr = (UINT16 *) group_ptr->dsp_param_ptr;
	display_decimal_number_in_mode(*word_ptr, mode);
}

void display_in_off_decimal_units( EDIT_OBJECT *group_ptr, BYTE mode)
{
	if (*group_ptr->dsp_param_ptr)
	{
		display_decimal_number_in_mode((UINT16) *group_ptr->dsp_param_ptr, mode);
	}
	else
		write_string_rj( "Off", mode );
		//write_string_rj( "None"/*"Off"*/, mode );
}

void display_in_mix_units( EDIT_OBJECT *group_ptr, BYTE mode)
{
#ifndef TEST_THIS_OUT
	char	temp_buf_2[5];
#endif
	BYTE 	num_position;
	BYTE	percent_position;

	//MessageBox(NULL, "mix_units", "Wow", MB_OK | MB_ICONINFORMATION);
	create_decimal_number((UINT16) *group_ptr->dsp_param_ptr, temp_buf);
	copy_string("%", temp_buf_2);

	percent_position = group_ptr->position;
	num_position = group_ptr->position-1;

	if (mode == MIDI_TRANSMIT_MODE)
	{
		string_cat( temp_buf, temp_buf_2);
		write_string_rj(temp_buf, mode);
	}
	else
	{
		if (mode == LONG_MODE)	
		{
			percent_position -= 4;
			num_position -= 4;
		}
			
	 	set_position(num_position, mode);
		write_string_rj( temp_buf, mode);
	 	set_position(percent_position, SHORT_MODE);
		write_string_rj( temp_buf_2, SHORT_MODE);

		if (mode == LONG_MODE)	
		{
		 	set_position(group_ptr->position, SHORT_MODE);
			write_string_rj( " Wet", SHORT_MODE);
		}
	}
	//write_string_rj( " Wet", SHORT_MODE);
}


void display_in_degree_units( EDIT_OBJECT *group_ptr, BYTE mode)
{
	create_decimal_number((UINT16) *group_ptr->dsp_param_ptr, temp_buf);
	if (mode == LONG_MODE)	
		string_cat( temp_buf, " degrees");

	write_string_rj( temp_buf, mode);
}


void display_temp_and_temp_buf_2(EDIT_OBJECT *group_ptr, BYTE mode, UINT8 offset)
{
	if (mode == MIDI_TRANSMIT_MODE)
	{
		string_cat( temp_buf, temp_buf_2);		
		write_string_rj( temp_buf, mode);
	}
	else
	{
	 	set_position(group_ptr->position-offset, mode);
		write_string_rj( temp_buf, mode);
 		set_position(group_ptr->position, SHORT_MODE);
		write_string_rj( temp_buf_2, SHORT_MODE);
	}
}



void display_value_in_level_units( EDIT_OBJECT *group_ptr, BYTE mode, BYTE val_mode)
{
	BYTE *min_ptr;
#ifndef TEST_THIS_OUT
	char temp_buf_2[5];
#endif

	*temp_buf_2 = 0;
	min_ptr = (BYTE *) &group_ptr->param_units.min_value; 

	if (*group_ptr->dsp_param_ptr == *min_ptr)
		write_string_rj( "Off", mode);
	else
	{
		if (*group_ptr->dsp_param_ptr)
			if (val_mode == 1)
			{
				// must be inverting the data
				INT8 data;
				data = (INT8) *group_ptr->dsp_param_ptr;
				//if (data > 0)
					data -= (data * 2);	// must be positive. Make negative
				//else
				//	data += (;	// must be negative. Make positive
				display_bipolar_byte(data, temp_buf);
			}
			else
				display_bipolar_byte(*group_ptr->dsp_param_ptr+(val_mode==2), temp_buf);
		else
			copy_string( "0", temp_buf);

		string_cat( temp_buf_2, "dB");

#ifndef TEST_THIS_OUT
		display_temp_and_temp_buf_2(group_ptr,mode,2);
#else
		if (mode == MIDI_TRANSMIT_MODE)
		{
			string_cat( temp_buf, temp_buf_2);		
			write_string_rj( temp_buf, mode);
		}
		else
		{
		 	set_position(group_ptr->position-2, mode);	// 2 == sizeof "dB"
			write_string_rj( temp_buf, mode);
	 		set_position(group_ptr->position, SHORT_MODE);
			write_string_rj( temp_buf_2, SHORT_MODE);
		}
#endif
	}
}

void display_in_level_units( EDIT_OBJECT *group_ptr, BYTE mode)
{
	display_value_in_level_units( group_ptr, mode, 0);
}

void display_in_inverted_level_units( EDIT_OBJECT *group_ptr, BYTE mode)
{
	display_value_in_level_units( group_ptr, mode, 1);
}

void display_in_level_2_units( EDIT_OBJECT *group_ptr, BYTE mode)
{
	display_value_in_level_units( group_ptr, mode, 2);
}



#ifdef schmutz
void display_in_cut_boost_level_units( EDIT_OBJECT *group_ptr, BYTE mode)
{
	char	data;	   

	data = (char) *group_ptr->dsp_param_ptr;

	data = data - 32;	// make 32 == 0

	if (data)
	{
		if (data & 0x80)
		{
			temp_buf[0] = '-';
			data = (0xff - data) + 1;
		}
		else
			temp_buf[0] = '+';

		temp_buf[1] = 0; /* null terminate */

		create_decimal_number((UINT16) data, &temp_buf[1] ); /* add number after sign */
	}
	else
		copy_string( "0", temp_buf);

 	set_position(group_ptr->position, mode);
	write_string_rj( temp_buf, mode);

}
#endif


void display_in_tone_level_units( EDIT_OBJECT *group_ptr, BYTE mode)
{
#ifndef TEST_THIS_OUT
	char	temp_buf_2[5];
#endif

	*temp_buf_2 = 0;
	
	if (*group_ptr->dsp_param_ptr)
		display_bipolar_byte(*group_ptr->dsp_param_ptr, temp_buf);
	else
		copy_string( "0", temp_buf);

	string_cat( temp_buf_2, "dB");

#ifndef TEST_THIS_OUT
	display_temp_and_temp_buf_2(group_ptr,mode,2);
#else
	if (mode == MIDI_TRANSMIT_MODE)
	{
		string_cat( temp_buf, temp_buf_2);		
		write_string_rj( temp_buf, mode);
	}
	else
	{
	 	set_position(group_ptr->position-2, mode);	// 2 == sizeof "dB"
		write_string_rj( temp_buf, mode);
 		set_position(group_ptr->position, SHORT_MODE);
		write_string_rj( temp_buf_2, SHORT_MODE);
	}
#endif
}


static void display_in_pitch_units( EDIT_OBJECT *group_ptr, BYTE mode, BYTE type)
{
	UINT16	data, *word_ptr;
	UINT16 	hi_digit;
	BYTE 	*data_ptr;
#ifndef TEST_THIS_OUT
	char	temp_buf_2[5];
#endif
	BYTE	hi_position;
	BYTE 	lo_position;
	BYTE	hi_mode, lo_mode;
	BYTE 	*buf_ptr;
	BYTE 	*buf_2_ptr;

	word_ptr = (UINT16 *) group_ptr->dsp_param_ptr;
	data = *word_ptr;
	buf_2_ptr = temp_buf_2;
	hi_digit = 0;

	if (type == 0)
	{
		if (data & 0x8000)
		{
			temp_buf[0] = '-';
			data = (0xffff - data) + 1;
		}
		else
			temp_buf[0] = '+';
	
		temp_buf[1] = 0; /* null terminate */
		buf_ptr = &temp_buf[1];
	}
	else
	{
		temp_buf[0] = 0; /* null terminate */
		buf_ptr = &temp_buf[0];
	}


	/* build the hi digits */
	while (data > 99)
	{
		++hi_digit;
		data -= 100;
	}		

	if (type == 2)
		create_decimal_number( hi_digit, buf_ptr ); /* add number after sign */

	if (hi_digit)
	{
		if (type != 2)
			create_decimal_number( hi_digit, buf_ptr ); /* add number after sign */

		/* build the lo digits */
	 	if (data < 10)
		{
	 		*temp_buf_2 = '0';
	 		buf_2_ptr = temp_buf_2+1;
		}
	}
	else if (type != 2)
 		string_cat( temp_buf, "_");  /* place holder */

	create_decimal_number( data, buf_2_ptr );
	
	if (mode == MIDI_TRANSMIT_MODE) 
	{
		if (type == 2)
			string_cat( temp_buf, ".");
		string_cat(temp_buf, temp_buf_2);
		write_string_rj(temp_buf, mode);
	}
	else
	{
		if (group_ptr->cur_field) /* must be low field */ 
		{
			lo_mode = mode;
			hi_mode = SHORT_MODE;
		}
		else
		{
			lo_mode = SHORT_MODE;
			hi_mode = mode;
		}

		if (mode == LONG_MODE)
		{
			hi_position = ((type==3) ? 19 : 25) - (string_len(temp_buf_2)+(type==2));
			lo_position	= ((type==3) ? 19 : 25);
		}
		else 
		{
			hi_position = group_ptr->position - (string_len(temp_buf_2)+(type==2));
			lo_position = group_ptr->position;
		}	

	 	set_position(hi_position, hi_mode);
		write_string_rj(temp_buf, hi_mode);

		if (type == 2)
		{
		 	set_position(hi_position + 1, SHORT_MODE);
			write_string_rj( ".", SHORT_MODE);
		}

	 	set_position(lo_position, lo_mode);
		write_string_rj(temp_buf_2, lo_mode);
	}
}

void display_in_coarse_pitch_units( EDIT_OBJECT *group_ptr, BYTE mode)
{
	display_in_pitch_units( group_ptr, mode, 0);
}

void display_in_Hz_units( EDIT_OBJECT *group_ptr, BYTE mode)
{
	display_in_pitch_units( group_ptr, mode, 1);
}




void display_in_name_units( EDIT_OBJECT *group_ptr, BYTE mode)
{
	char *string_ptr;
	BYTE offset;
	BYTE position;

	position 	= group_ptr->position;
	offset 		= group_ptr->cur_field;

#ifdef EMBEDDED_Z80_CODE	
	if (cur_prog_ptr == &original_prog)
		string_ptr 	= group_ptr->original_param_ptr;
	else
#endif
		string_ptr 	= group_ptr->param_ptr;

	/* broken??? */
 	set_position(position, SHORT_MODE);
	write_x_chars_of_string_rj( string_ptr,	group_ptr->param_desc.num_fields, SHORT_MODE);

/*	set_position(offset + position, mode);*/
	set_position(position - (group_ptr->param_desc.num_fields - (offset+1)), mode);
	write_char(string_ptr[offset], mode);
}

void display_in_program_name_units( EDIT_OBJECT *group_ptr, BYTE mode)
{
#ifdef EMBEDDED_Z80_CODE	
	UINT16	*prog_num_ptr;
	char	temp_buf_3[PROG_NAME_SIZE+1];

	prog_num_ptr = (UINT16 *) group_ptr->param_ptr;
	create_decimal_number(*prog_num_ptr, temp_buf);
	copy_program_name_to_buf(*prog_num_ptr, temp_buf_3);

	string_cat( temp_buf, " ");	
	string_cat( temp_buf, temp_buf_3);	// add program name to number

	write_x_chars_of_string_to_buf_position( temp_buf, group_ptr->position - (PROG_NAME_SIZE+1), PROG_NAME_SIZE);
#endif

}


void display_in_BPM_units( EDIT_OBJECT *group_ptr, BYTE mode)
{
	UINT16 *word_ptr;
#ifndef TEST_THIS_OUT
	char	temp_buf_2[5];
#endif

	word_ptr = (UINT16 *) group_ptr->dsp_param_ptr;

	create_decimal_number(*word_ptr, temp_buf);
	copy_string(" BPM", temp_buf_2);

#ifndef TEST_THIS_OUT
	display_temp_and_temp_buf_2(group_ptr,mode,4);
#else
	if (mode == MIDI_TRANSMIT_MODE)
	{
		string_cat( temp_buf, temp_buf_2);
		write_string_rj(temp_buf, mode);
	}
	else
	{
	 	set_position(group_ptr->position-4, mode);
		write_string_rj( temp_buf, mode);
	 	set_position(group_ptr->position, SHORT_MODE);
		write_string_rj( temp_buf_2, SHORT_MODE);
	}
#endif
}

void display_in_num_beats_units( EDIT_OBJECT *group_ptr, BYTE mode)
{
#ifndef TEST_THIS_OUT
	char	temp_buf_2[7];
#endif

	*temp_buf_2 = 0;

	if (*group_ptr->dsp_param_ptr <= beat_value_DOTTED_QUARTER_NOTE)
		write_string_rj( (char *) note_strings[*group_ptr->dsp_param_ptr], mode );
	else
	{
		create_decimal_number((UINT16) *group_ptr->dsp_param_ptr - 2, temp_buf);
		copy_string(" Beats", temp_buf_2);

#ifndef TEST_THIS_OUT
		display_temp_and_temp_buf_2(group_ptr,mode,6);
#else
		if (mode == MIDI_TRANSMIT_MODE)
		{
			string_cat( temp_buf, temp_buf_2);
			write_string_rj(temp_buf, mode);
		}
		else
		{
		 	set_position(group_ptr->position-6, mode);
			write_string_rj( temp_buf, mode);
		 	set_position(group_ptr->position, SHORT_MODE);
			write_string_rj( temp_buf_2, SHORT_MODE);
		}
#endif
	}
}

static void display_in_tuner_x_units( EDIT_OBJECT *group_ptr, BYTE mode, BYTE type)
{
	char	temp_buf_3[30];
	BYTE position;

	if (!type)
	{
		copy_string("A=", temp_buf_3);
		copy_string( (char *) tuner_cal_strings[*group_ptr->dsp_param_ptr], temp_buf);
	}
	else
	{
		//create_decimal_number((UINT16) *group_ptr->dsp_param_ptr, temp_buf);
		display_bipolar_byte(*group_ptr->dsp_param_ptr, temp_buf);
		copy_string("Cents = ", temp_buf_3);
	}

	if (mode == MIDI_TRANSMIT_MODE)
	{
		string_cat( temp_buf_3, temp_buf);
		write_string_rj(temp_buf_3, mode);
	}
	else
	{
		position = group_ptr->position - string_len(temp_buf);
	 	set_position(position, SHORT_MODE);
		write_string_rj( temp_buf_3, SHORT_MODE);

	 	set_position(group_ptr->position, mode);
		write_string_rj( temp_buf, mode);
	}
}

void display_in_tuner_cal_units( EDIT_OBJECT *group_ptr, BYTE mode)
{
	display_in_tuner_x_units( group_ptr, mode, 0);
}

void display_in_tuner_offset_units( EDIT_OBJECT *group_ptr, BYTE mode)
{
	display_in_tuner_x_units( group_ptr, mode, 1);
}


//void display_tuner_offset( EDIT_OBJECT *group_ptr, BYTE position)
//{
//	set_position(position, SHORT_MODE);
//	copy_string( tuner_offset_strings[*group_ptr->dsp_param_ptr], temp_buf );
//	write_string_rj( temp_buf, SHORT_MODE);
//}



void display_in_percentage_units( EDIT_OBJECT *group_ptr, BYTE mode)
{
#ifndef TEST_THIS_OUT
	char	temp_buf_2[9];
#endif
	BYTE 	position_offset;

	create_decimal_number((UINT16) *group_ptr->dsp_param_ptr, temp_buf);

//	if (mode == LONG_MODE)
//	{
//		copy_string(" percent", temp_buf_2);
//		position_offset = 8;
//	}
//	else
	{
		copy_string("%", temp_buf_2);
		position_offset = 1;
	}

#ifndef TEST_THIS_OUT
	display_temp_and_temp_buf_2(group_ptr,mode,position_offset);
#else
	if (mode == MIDI_TRANSMIT_MODE)
	{
		string_cat( temp_buf, temp_buf_2);
		write_string_rj(temp_buf, mode);
	}
	else
	{
	 	set_position(group_ptr->position - position_offset, mode);
		write_string_rj( temp_buf, mode);
	 	set_position(group_ptr->position, SHORT_MODE);
		write_string_rj( temp_buf_2, SHORT_MODE);
	}
#endif
}

void display_in_bipolar_percentage_units( EDIT_OBJECT *group_ptr, BYTE mode)
{
#ifndef TEST_THIS_OUT
	char	temp_buf_2[5];
#endif

	display_bipolar_byte(*group_ptr->dsp_param_ptr, temp_buf);
	copy_string("%", temp_buf_2);

#ifndef TEST_THIS_OUT
	display_temp_and_temp_buf_2(group_ptr,mode,1);
#else
	if (mode == MIDI_TRANSMIT_MODE)
	{
		string_cat( temp_buf, temp_buf_2);
		write_string_rj(temp_buf, mode);
	}
	else
	{
	 	set_position(group_ptr->position-1, mode);
		write_string_rj( temp_buf, mode);
	 	set_position(group_ptr->position, SHORT_MODE);
		write_string_rj( temp_buf_2, SHORT_MODE);
	}
#endif
}


void display_in_bipolar_degrees_units( EDIT_OBJECT *group_ptr, BYTE mode)
{
	display_bipolar_byte(*group_ptr->dsp_param_ptr, temp_buf);
	if (mode == LONG_MODE)
		string_cat( temp_buf, " degrees");

	write_string_rj( temp_buf, mode);
}


void display_in_Q_units( EDIT_OBJECT *group_ptr, BYTE mode)
{
	BYTE data;
	BYTE hi_digit;
#ifndef TEST_THIS_OUT
	BYTE temp_buf_2[5];
#endif

	data = *group_ptr->dsp_param_ptr;
	hi_digit = 0;

	set_position(group_ptr->position-1, mode);

	while(data > 9)
	{
		++hi_digit;
		data -= 10;
	}		

	create_decimal_number(hi_digit, temp_buf); 
	string_cat( temp_buf, ".");
	create_decimal_number(data, temp_buf_2); 
	string_cat( temp_buf, temp_buf_2);

	if (mode == LONG_MODE)	
		string_cat( temp_buf, " Q");

	write_string_rj( temp_buf, mode);
}


/* display routines */

extern void get_param_name(BYTE effect_type, BYTE param_index, BYTE *buf_ptr);
extern void get_effect_name(BYTE effect_type, BYTE *buf_ptr);
extern void get_level_b_string(BYTE effect_type, BYTE *buf_ptr);

void display_in_patch_dest_units( EDIT_OBJECT *group_ptr, BYTE mode)
{
	BYTE data;
	BYTE effect_type;
	BYTE param_index;
	//BYTE position;
#ifdef EMBEDDED_Z80_CODE
	BYTE temp_buf_3[15];
#endif
	
	data = *group_ptr->dsp_param_ptr;
	if (data == NO_DEST)
		write_string_rj( "Unassignd", mode);
	else
	{
		//position = group_ptr->position;
#ifdef EMBEDDED_Z80_CODE	
		effect_type = cur_prog_ptr->patches[cur_choices.cur_patch_num].dest.effect_type;
		param_index = cur_prog_ptr->patches[cur_choices.cur_patch_num].dest.param_index;

		get_level_b_string(effect_type, temp_buf);
		get_param_name(effect_type, param_index, temp_buf_3);
		string_cat(temp_buf, temp_buf_3);
		write_string_rj( temp_buf, mode);
#endif
		//midi_puts("patch:");
		//midi_display_decimal_number(effect_type);
		//midi_display_decimal_number(param_index);
	}
}

void display_in_soft_row_assign_units( EDIT_OBJECT *group_ptr, BYTE mode)
{
	BYTE data;
	BYTE effect_type;
	BYTE param_index;
	//BYTE position;
#ifndef TEST_THIS_OUT
	BYTE temp_buf_2[10];
#endif
	
#ifdef EMBEDDED_Z80_CODE	
	data = *group_ptr->dsp_param_ptr;
	effect_type = cur_prog_ptr->soft_rows[cur_choices.soft_row_edit].effect_type;
	param_index = cur_prog_ptr->soft_rows[cur_choices.soft_row_edit].param_index;

	if (data == NO_DEST || param_index == UNASSIGNED || effect_type == UNASSIGNED)
		write_string_rj( "Unassignd", mode);
	else
	{
		//position = group_ptr->position;

		get_level_b_string(effect_type, temp_buf);
		get_param_name(effect_type, param_index, temp_buf_2);
		string_cat(temp_buf, temp_buf_2);
		write_string_rj( temp_buf, mode);
	}
#endif
}


void display_internal_controller(BYTE cont_num, BYTE position, BYTE mode)
{
	set_position(position, mode);
	write_string_rj( (char *) internal_controller_name_strings[cont_num-1], mode);
}

void display_midi_controller_number(BYTE cont_num, BYTE position, BYTE mode)
{
	BYTE value;

	copy_string("CC", temp_buf);
	set_position(position, mode);
	value = cont_num - (NUM_INT_CONTROLLERS-1);
	if (value >= 32)
		++value; 
	create_decimal_number(value, &temp_buf[2] );

	write_string_rj( temp_buf, mode);
}

void display_midi_control_number(BYTE cont_num, BYTE position, BYTE mode)
{
#ifdef EMBEDDED_Z80_CODE	
	write_custom_char_to_lcd_ram(&note_char, SECOND_CUSTOM_CHAR);
#endif
	set_position(position, mode);
 	write_string_rj( (char *)  midi_control_name_strings[cont_num-FIRST_MIDI_CONTROL], mode);
}

void display_in_cont_group_and_source_units( EDIT_OBJECT *group_ptr, BYTE mode)
//void draw_controller_source(BYTE cont_num, BYTE position, BYTE mode)
{
	BYTE cont_num;
	BYTE position;
	 
	cont_num = *group_ptr->dsp_param_ptr;
	position = group_ptr->position;

	if ( cont_num == group_ptr->param_units.min_value)
//	if (cont_num == NO_CONT)
	{
		set_position(position, mode);
		write_string_rj( "Unassignd", mode);
		return;
	}

	set_position(position-8, SHORT_MODE);
	if (cont_num < NUM_INT_CONTROLLERS)
	{
		write_string("Ctls", SHORT_MODE);
		display_internal_controller(cont_num, position, mode);
	}
	else
	{
		write_string("MIDI", SHORT_MODE);
		if (cont_num >= FIRST_MIDI_CONTROLLER && cont_num < FIRST_MIDI_CONTROL)
			display_midi_controller_number(cont_num, position, mode);
		else 
			display_midi_control_number(cont_num, position, mode);
	}
}


void display_in_cont_source_units( EDIT_OBJECT *group_ptr, BYTE mode)
{
	BYTE cont_num;

/*	cont_num = *group_ptr->dsp_param_ptr;*/
	cont_num = *group_ptr->param_ptr;

	//if ( cont_num == SOURCE_UNASSIGNED || cont_num == 0)
	if ( cont_num == group_ptr->param_units.min_value)
	{
		 write_string_rj( "None", mode);
	}
	else if (cont_num < NUM_INT_CONTROLLERS)
	{
		display_internal_controller(cont_num, group_ptr->position, mode);
	}
	else if (cont_num >= FIRST_MIDI_CONTROLLER && cont_num < FIRST_MIDI_CONTROL)
	{
		display_midi_controller_number(cont_num, group_ptr->position, mode);
	}										
	else 
	{
		display_midi_control_number(cont_num, group_ptr->position, mode);
	}
}


void display_in_min_off_cont_source_units( EDIT_OBJECT *group_ptr, BYTE mode)
{
	BYTE cont_num;

/*	cont_num = *group_ptr->dsp_param_ptr;*/
	cont_num = *group_ptr->param_ptr;

	if (   cont_num == group_ptr->param_units.min_value )
			write_string_rj( "None", mode);
	else
		display_in_cont_source_units( group_ptr, mode);
}

void display_in_arp_velocity_source_units( EDIT_OBJECT *group_ptr, BYTE mode)
{
	BYTE cont_num;

	cont_num = *group_ptr->param_ptr;
	if (cont_num <= NUM_CONST_VELOCITY_SOURCES)
	{
		create_decimal_number( cont_num, temp_buf);
		write_string_rj( temp_buf, mode );
	}
	else
	{
		/* we need to subtract the offset from the param value when using this function */
		*group_ptr->param_ptr = cont_num - NUM_CONST_VELOCITY_SOURCES;
		display_in_cont_source_units( group_ptr, mode);
		*group_ptr->param_ptr = cont_num;
	}		
}		 
	

void display_in_rate_unit_units( EDIT_OBJECT *group_ptr, BYTE mode)
{
	UINT16	data;
	UINT16 	*word_ptr;

/*	word_ptr = (UINT16 *) group_ptr->dsp_param_ptr;*/
//	word_ptr = (UINT16 *) group_ptr->param_ptr;	/* the patch messes up the value */
//	data = *word_ptr;
//	if (data & 0x8000)
	if (*group_ptr->option_ptr)
	{
		if (mode == LONG_MODE)
			write_string_rj( "cycles:beat", mode);
		else
			write_string_rj( "cy:bt", mode);
	}
	else
		write_string_rj( "Hz", mode);
}
	  

void display_in_rate_units( EDIT_OBJECT *group_ptr, BYTE mode)
{
	display_in_pitch_units( group_ptr, mode, 2);
}

void display_in_ms_units( EDIT_OBJECT *group_ptr, BYTE mode)
{
	if (group_ptr->param_desc.num_bytes == 1)
		display_decimal_number_in_mode((UINT16) *group_ptr->dsp_param_ptr, mode);
	else
		display_in_pitch_units( group_ptr, mode, 3);
}


void display_in_l_units( EDIT_OBJECT *group_ptr, BYTE mode, BYTE type)
{
	UINT16 *word_ptr;

	word_ptr = (UINT16 *) group_ptr->dsp_param_ptr;
	create_decimal_number( *word_ptr, temp_buf );

	if (mode == LONG_MODE)
	{
	 	set_position(31, SHORT_MODE);
		if (!type)
		{
			write_string_rj("Feet", SHORT_MODE);
		 	set_position(26, SHORT_FLASHING_MODE);
		}
		else
		{
			write_string_rj("Meters", SHORT_MODE);
		 	set_position(24, SHORT_FLASHING_MODE);
		}
		write_string_rj(temp_buf, SHORT_FLASHING_MODE);
	}
	else
		write_string_rj(temp_buf, mode);
}



void display_in_feet_units( EDIT_OBJECT *group_ptr, BYTE mode)
{
	display_in_l_units( group_ptr, mode, 0);
}


void display_in_meter_units( EDIT_OBJECT *group_ptr, BYTE mode)
{
	display_in_l_units( group_ptr, mode, 1);
}



void display_in_time_sig_units( EDIT_OBJECT *group_ptr, BYTE mode)
{
	UINT16	data, *word_ptr;
	UINT16 	hi_digit;
	BYTE 	*data_ptr;
	BYTE	hi_mode, lo_mode;
#ifndef TEST_THIS_OUT
	char	temp_buf_2[5];
#endif
	BYTE 	*buf_2_ptr;
	BYTE	hi_position;
	BYTE 	lo_position;


	word_ptr = (UINT16 *) group_ptr->dsp_param_ptr;
	data = *word_ptr;
	hi_digit = 0;

	data_ptr = group_ptr->dsp_param_ptr;

	create_decimal_number( (UINT16) *data_ptr, temp_buf); /* cycles */

	++data_ptr;
	create_decimal_number( (UINT16) (*data_ptr & 0x7f), temp_buf_2 ); /* beat */


	if (mode == MIDI_TRANSMIT_MODE)
	{
		string_cat( temp_buf, ":");
		string_cat(temp_buf, temp_buf_2);
		write_string_rj(temp_buf, mode);
	}
	else
	{
		if (group_ptr->cur_field) /* must be low field */ 
		{
			lo_mode = mode;
			hi_mode = SHORT_MODE;
		}
		else
		{
			lo_mode = SHORT_MODE;
			hi_mode = mode;
		}

		if (mode == LONG_MODE)
		{
			hi_position = 20 - (string_len(temp_buf_2) + 1);
			lo_position	= 20;
		}
		else 
		{
			hi_position = group_ptr->position - (string_len(temp_buf_2) + 1); /* the : */
			lo_position = group_ptr->position;
		}	

	 	set_position(hi_position, hi_mode);
		write_string_rj(temp_buf, hi_mode);

	 	set_position(hi_position + 1, SHORT_MODE);
		write_string_rj( ":", SHORT_MODE);

	 	set_position(lo_position, lo_mode);
		write_string_rj(temp_buf_2, lo_mode);

//		if (mode == LONG_MODE)
//		{
//			if (group_ptr->effect_desc.type == DELAY_EFFECT)
//				write_string_to_buf_position( " echo:beat", 21);
//			else
//				write_string_to_buf_position( " cycle:beat", 21);
//		}
	}

}

#ifdef schmutz
		if (mode == LONG_MODE)	
		{
			switch (*group_ptr->option_ptr)
			{
				case EQ_IN_MS:
					break;
				case EQ_IN_FEET:
					write_string_to_buf( " Feet");
					break;
				case EQ_IN_METERS:
					write_string_to_buf( " Meters");
					break;
				case EQ_IN_TAPPED_TIME:
#pragma warnings=off  /* don't complain about unprintable chars */
					write_string_to_buf( "ms "); /* the note symbol */
#pragma warnings=on 
					break;
			}
		}

	}

}
#endif


void display_in_midi_channel_units( EDIT_OBJECT *group_ptr, BYTE mode)
{
	BYTE 	static_mode;

	if (mode == MIDI_TRANSMIT_MODE)
		static_mode = MIDI_TRANSMIT_MODE;
	else
		static_mode = SHORT_MODE;

	switch (*group_ptr->dsp_param_ptr)
	{
		case OMNI_MODE: 
			set_position(group_ptr->position, mode);
			write_string( "Omni", mode);
			break;
		case MIDI_OFF:	
			set_position(group_ptr->position, mode);
			write_string( "Off", mode);
			break;
		default:
			set_position(group_ptr->position, static_mode);
			write_string( "chan = ", static_mode);
			set_position(group_ptr->position + 7, mode);
			display_decimal_number_in_mode( (UINT16) *group_ptr->dsp_param_ptr + 1, 
												mode );
			break;
	}

}

void display_control_name_units( EDIT_OBJECT *group_ptr, BYTE mode)
{
	set_position(group_ptr->position, mode);
	write_x_chars_of_string_rj(group_ptr->effect_desc.name, EFFECT_NAME_SIZE, mode);
}

void display_parameter_name_units( EDIT_OBJECT *group_ptr, BYTE mode)
{
	BYTE size;

	//if (group_ptr->param_type <= LAST_AUDIO_PARAM)
	if (group_ptr->param_desc.control_flags & AUDIO_PARAM_FLAG)// only Audio params have options
		size = PARAM_NAME_SIZE;
	else
		size = SETUP_PARAM_NAME_SIZE;

	write_x_chars_of_string_rj(group_ptr->effect_desc.name, EFFECT_NAME_SIZE, mode);

#ifdef schmutz
	if (mode == MIDI_TRANSMIT_MODE)
	{
		transmit_nibblized_word(size);			/* number of bytes in string */
		write_x_chars_of_string( group_ptr->param_desc.name, size, mode);
	}
	else
		write_x_chars_of_string_to_buf_position_rj( group_ptr->param_desc.name, group_ptr->position, size);
#endif
}




